package PaperlessOffice::Document::ImagePDF;

use base qw(PaperlessOffice::Document);

use Manager::Dialog qw(ApproveCommands Message);
use PerlLib::SwissArmyKnife;

use Data::Dumper;
use File::Slurp;
use File::Stat;
use IO::File;

use Class::MethodMaker
  new_with_init => 'new',
  get_set       =>
  [

   qw / /

  ];

sub init {
  my ($self,%args) = @_;
  print "PaperlessOffice::Document::ImagePDF->init\n" if $UNIVERSAL::paperlessoffice->Debug;

  $self->PageImages($args{PageImages} || []);
  $self->PageTexts($args{PageTexts} || []);

  $self->SUPER::init(%args);
  my $dir = $self->SUPER::Dir;
  my $quoteddir = $self->SUPER::QuotedDir;

  my $res = $UNIVERSAL::paperlessoffice->MyDocumentManager->GetMetadata
    (
     DocumentID => $self->DocumentID,
     Predicate => "has-pdf",
    );
  if ($res->{Success}) {
    if ($res->{Result} eq "__unknown") {
      print "Big problem with: ".$self->DocumentID."\n";
    } else {
      $self->SUPER::TargetFile($res->{Result});
    }
  } else {
    my $res = `ls -1 $quoteddir | grep -ir pdf`;
    if ($res =~ /^(.+\.pdf)$/im) {
      my $pdffile = $1;
      my $res = $UNIVERSAL::paperlessoffice->MyDocumentManager->SetMetadata
	(
	 DocumentID => $self->DocumentID,
	 Predicate => "has-pdf",
	 Value => $pdffile,
	);
      $self->SUPER::TargetFile($pdffile);
    } else {
      my $res = $UNIVERSAL::paperlessoffice->MyDocumentManager->SetMetadata
	(
	 DocumentID => $self->DocumentID,
	 Predicate => "has-pdf",
	 Value => "__unknown",
	);
    }
  }

  my $file = $self->SUPER::TargetFile;
  my $quotedtargetfile = shell_quote($file);
  $self->SUPER::QuotedTargetFile($quotedtargetfile);

  if (0 and ! -f "$dir/full.txt") {
    my $c = "cd $quoteddir && pdftoppm $quotedfile convert";
    ApproveCommands
      (
       Commands => [$c],
       Method => "parallel",
       # AutoApprove => 1,
      );

    # now convert the filenames
    my $i = 1;
    my $j = 1;
    my $exitj = 0;
    while (! $exitj) {
      my $exiti = 0;
      while (! $exiti) {
	my $file = sprintf("%s/convert-%0".$j."i.ppm",$quoteddir,$i);
	if (! -f $file) {
	  $exiti = 1;
	} else {
	  my $fn = "$quoteddir/".($i-1).".pnm";
	  my $qfile = shell_quote($file);
	  my $qfn = shell_quote($fn);
	  my $c2 = "mv $qfile $qfn";
	  print "PaperlessOffice::Document::ImagePDF->init 2\n";
	  ApproveCommands
	    (
	     Commands => [$c2],
	     Method => "parallel",
	     # AutoApprove => 1,
	    );
	  $doc->PageImages->[$i-1] = $fn;
	  ++$i;
	  $exitj = 1;
	}
      }
      if ($j > 9) {
	$exitj = 1;
      }
      ++$j;
    }
    $self->OCR;
  }
}

sub AddImage {
  my ($self,%args) = @_;
  if (-f $args{Image}) {
    push @{$self->PageImages}, $args{Image};
  }
}

sub OCR {
  my ($self,%args) = @_;
  my $last = scalar @{$self->PageImages} - 1;
  foreach my $i (0..$last) {
    my $fn = $self->PageImages->[$i].".txt";
    my $fn2 = $self->PageImages->[$i].".badresults.txt";
    if (! -f $fn) {
      print "OCRing page ".($i + 1)."\n";
      my $res = $UNIVERSAL::paperlessoffice->MyResources->MyOCR->OCRImage
	(ImageFile => $self->PageImages->[$i]);
      if (scalar @{$res->{BadResults}}) {
	my $fh2 = IO::File->new;
	if ($fh2->open(">$fn2")) {
	  print $fh2 Dumper($res->{BadResults});
	  $fh2->close;
	} else {
	  print "cannot open file <$fn2> for writing OCR badresults output\n";
	}
      }
      if ($res->{Success}) {
	$self->PageTexts->[$i] = $res->{Text};
	my $fh = IO::File->new;
	if ($fh->open(">$fn")) {
	  print $fh $res->{Text};
	  $fh->close;
	} else {
	  print "cannot open file <$fn> for writing OCR output\n";
	}
      } else {
	$self->PageTexts->[$i] = undef;
      }
    }
  }
  $self->SUPER::OCRedP(1);
}

sub IsEmptyAndShouldBeDeleted {
  my ($self,%args) = @_;
  return 0;
  #   my $count = scalar @{$self->PageImages};
  #   if ($count == 0) {
  #     return 1;
  #   } elsif ($count == 1) {
  #     my $stat =  File::Stat->new($self->PageImages->[0]);
  #     return ($stat->size == 0);
  #   }
}

# all sorts of other glorious functions here

sub FullText {
  my ($self,%args) = @_;
  my @contents;
  for (my $i = 0; $i < scalar @{$self->PageTexts}; ++$i) {
    if (exists $self->PageTexts->[$i]) {
      my $f = $self->PageTexts->[$i];
      if (-f $f) {
	push @contents, read_file($f);
      }
    }
  }
  return join("\n",@contents);
}

sub GeneratePDF {
  my ($self,%args) = @_;

}

sub GenerateThumbnail {
  my ($self,%args) = @_;
  my $thumbnailfile = ConcatDir($self->Directory,"thumbnail.gif");
  if (! -f $thumbnailfile or $args{Regenerate}) {
    my $quotedppmfile = shell_quote(ConcatDir($self->Directory,"output-1.ppm"));
    my $res = ApproveCommands
      (
       Commands =>
       [
	"cd ".shell_quote($self->Directory)." && pdftoppm ".shell_quote($file)." -f 1 -l 1 output",
	"convert -thumbnail 100 $quotedppmfile ".shell_quote($thumbnailfile),
	"rm $quotedppmfile",
       ],
       Method => "parallel",
       AutoApprove => $args{AutoApprove} || 1,
      );
    if ($res) {
      if (-f $thumbnailfile) {
	# assert this as the thumbnail file
	my $res = $UNIVERSAL::paperlessoffice->MyDocumentManager->SetMetadata
	  (
	   DocumentID => $self->DocumentID,
	   Predicate => "has-thumbnail",
	   Value => "thumbnail.gif",
	  );
      } else {
	# assert that the thumbnail file could not be generated
	my $res = $UNIVERSAL::paperlessoffice->MyDocumentManager->SetMetadata
	  (
	   DocumentID => $self->DocumentID,
	   Predicate => "has-thumbnail",
	   Value => "Generation failed",
	  );
      }
    } else {
      # assert that we haven't attempted to generate the thumbnailfile
	my $res = $UNIVERSAL::paperlessoffice->MyDocumentManager->SetMetadata
	  (
	   DocumentID => $self->DocumentID,
	   Predicate => "has-thumbnail",
	   Value => "Not yet generated",
	  );
    }
  } else {
    my $res = $UNIVERSAL::paperlessoffice->MyDocumentManager->SetMetadata
      (
       DocumentID => $self->DocumentID,
       Predicate => "has-thumbnail",
       Value => "thumbnail.gif",
      );
  }
}

sub GetThumbnail {
  my ($self,%args) = @_;
  $self->SUPER::GetThumbnail(%args);
}

sub MenuActionView {
  my ($self,%args) = @_;
  my $command = "xdg-open ".shell_quote(ConcatDir($self->Directory,$self->TargetFile))." &";
  ApproveCommands
    (
     Commands => [$command],
     Method => "parallel",
     AutoApprove => 1,
    );
}

sub MenuActionViewText {
  my ($self,%args) = @_;
  my $fh = IO::File->new();
  my $tmpfile = "/tmp/paperless-office/doc.txt";
  $fh->open(">$tmpfile") or warn "cannot open";
  print $fh $self->FullText;
  $fh->close();
  ApproveCommands
    (
     Commands => [
		  "gedit ".shell_quote($tmpfile)." &",
		 ],
     Method => "parallel",
     AutoApprove => 1,
    );
}

sub MenuActionEdit {
  my ($self,%args) = @_;
  my $command = "pdfedit ".shell_quote(ConcatDir($self->Directory,$self->TargetFile))." &";
  ApproveCommands
    (
     Commands => [$command],
     Method => "parallel",
     AutoApprove => 1,
    );
}

sub MenuActionFormFiller {
  my ($self,%args) = @_;
  my $command = "flpsed ".shell_quote(ConcatDir($self->Directory,$self->TargetFile))." &";
  ApproveCommands
    (
     Commands => [$command],
     Method => "parallel",
     AutoApprove => 1,
    );
}


1;
